/*-----------------------------------------------------------------------------
//
//    TNKernel startup file  for ARM processors(running from FLASH)
//
//
// Copyright  2004,2005 Yuri Tiomkin
// All rights reserved.
//
// Permission to use, copy, modify, and distribute this software in source
// and binary forms and its documentation for any purpose and without fee
// is hereby granted, provided that the above copyright notice appear
// in all copies and that both that copyright notice and this permission
// notice appear in supporting documentation.
//
// THIS SOFTWARE IS PROVIDED BY THE YURI TIOMKIN AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL YURI TIOMKIN OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//-----------------------------------------------------------------------------*/


    .equ MODE_BITS,   0x1F               /* Bit mask for mode bits in CPSR */
    .equ USR_MODE,    0x10               /* User mode */
    .equ FIQ_MODE,    0x11               /* Fast Interrupt Request mode */
    .equ IRQ_MODE,    0x12               /* Interrupt Request mode */
    .equ SVC_MODE,    0x13               /* Supervisor mode */
    .equ ABT_MODE,    0x17               /* Abort mode */
    .equ UND_MODE,    0x1B               /* Undefined Instruction mode */
    .equ SYS_MODE,    0x1F               /* System mode */

    .set I_Bit, 0x80    /* when I bit is set, IRQ is disabled */
    .set F_Bit, 0x40    /* when F bit is set, FIQ is disabled */

    .section    .reset, "ax"
    .global  __reset
    .code 32

__reset:
        ldr    pc, reset_handler_address
        ldr    pc, undef_handler_address
        ldr    pc, swi_handler_address
        ldr    pc, pabort_handler_address
        ldr    pc, dabort_handler_address
//        .word  0xb9205f80                /* Wie generiert man die Prfsumme automatisch ? */
//        ldr    pc,[pc, #-0x0ff0]         /* Dies funktioniert nicht */
        .word  0xB8A06F58                  /* 0 - (sum of other vectors instructions) */
        ldr    pc, irq_handler_address
        ldr    pc, fiq_handler_address

reset_handler_address:   .word  reset_handler
undef_handler_address:   .word  undef_handler
swi_handler_address:     .word  swi_handler
pabort_handler_address:  .word  pabort_handler
dabort_handler_address:  .word  dabort_handler
                         .word  0
irq_handler_address:     .word  irq_handler
fiq_handler_address:     .word  fiq_handler
                         .word  0
                         .word  0

    .text
    .code 32
    .align 0

undef_handler:
        b      undef_handler
swi_handler:
        b      swi_handler
pabort_handler:
        b      pabort_handler
dabort_handler:
        b      dabort_handler
irq_handler:
        b      _irq_handler
fiq_handler:
        b      fiq_handler

/*
 * Some processor registers
 */
rMAMTIM:    .word  0xE01FC004
rMAMCR:     .word  0xE01FC000

/* ------------------------- */
reset_handler:
        // Flash speed
        // rMAMTIM = 3
        ldr    r0, rMAMTIM
        mov    r1, #3             // MAM fetch cycles are 3 CCLK in duration
        strb   r1, [r0]

        // rMAMCR  = 2
        ldr    r0, rMAMCR
        mov    r1, #2             // MAM functions fully enabled
        strb   r1, [r0]

        // init stacks
        mrs    r0,cpsr            // Original PSR value

        bic    r0,r0,#MODE_BITS   // Clear the mode bits
        orr    r0,r0,#IRQ_MODE    // Set IRQ mode bits
        msr    cpsr_c,r0          // Change the mode
        ldr    sp, stack_irq_end  // End of IRQ_STACK

        bic    r0,r0,#MODE_BITS   // Clear the mode bits
        orr    r0,r0,#FIQ_MODE    // Set FIQ mode bits
        msr    cpsr_c,r0          // Change the mode
        ldr    sp, stack_fiq_end  // End of FIQ_STACK

        bic    r0,r0,#MODE_BITS   // Clear the mode bits
        orr    r0,r0,#SVC_MODE    // Set Supervisor mode bits
        msr    cpsr_c,r0          // Change the mode
        ldr    sp, stack_end      // End of stack

        /*-- Leave core in SVC mode ! */

        // Clear BSS (zero init)
.extern     __bss_start
.extern     __bss_end__

        mov   r0,#0
        ldr   r1,=__bss_start
        ldr   r2,=__bss_end__
2:      cmp   r1,r2
        strlo r0,[r1],#4
        blo   2b

        // Copy Initialized data from FLASH to RAM
.extern  _etext
.extern  _data
.extern  _edata

        ldr   r1,=_etext
        ldr   r2,=_data
        ldr   r3,=_edata
1:      cmp   r2,r3
        ldrlo r0,[r1],#4
        strlo r0,[r2],#4
        blo   1b

        // call main
.extern   main

        ldr    r0,=main
        mov    pc, r0                        /*  goto main() */

        // you should never return from main.
        // if you do anyway, loop forever.
exit_loop:
        b      exit_loop

/*
 * some helper functions
 */
.equ      NOINT,     0xc0
.global   disable_interrupts
.global   enable_interrupts

disable_interrupts:
        mrs    r0, cpsr
        orr    r0, r0, #NOINT
        msr    cpsr_c, r0
        bx     lr

enable_interrupts:
        mrs    r0, cpsr
        bic    r0, r0, #NOINT
        msr    cpsr_c, r0
        bx     lr

/*--- IRQ Handler ----------------------*/
.set VIC_base_addr, 0xFFFFF000
.set VIC_vect_offs, 0x30

_irq_handler:
        stmfd  sp!,{r0-r12,lr}              // save ALL registers
        ldr    r14, =VIC_base_addr          // get VIC's base address
        ldr    r0 , [r14, #VIC_vect_offs]   // load interrupt vector
        mov    r14, pc                      // save program counter
        bx     r0                           // jump to interrupt vector
        ldmfd  sp!,{r0-r12,lr}              // restore registers
        subs   pc,lr,#4                     // return from IRQ
/*--------------------------------------*/

/*--------------------------  */
.extern   __stack_irq_bottom_end__
.extern   __stack_fiq_bottom_end__
.extern   __stack_bottom_end__

stack_end:      .word   __stack_irq_bottom_end__
stack_irq_end:  .word   __stack_fiq_bottom_end__
stack_fiq_end:  .word   __stack_bottom_end__

/*-----------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------*/


