/**
 * TLC5940 Driver library for LPC2148
 * (C) Thorsten Erdmann (thorsten@trektech.de)
 **/
#include <stdio.h>
#include "lpc2148.h"
#include "tlc5940.h"

/**
 * PWM1 interrupt routine used for BLANK generation
 **/

void pwm0ISR(void)
{
    uint16_t temp=PWMIR;
    if (temp & PWMIR_PWMMR0_Interrupt)
//    if (PWMIR & PWMIR_PWMMR0_Interrupt)
    {
        T1TCR   = 0;     // stop timer 1 (GSCLK)
        FIO0SET = BLANK;
        FIO0SET = XLAT;
        //temp=temp & 1;     // eat a bit of time
        FIO0CLR = XLAT;
        FIO0CLR = BLANK;
        T1TCR    = 0x02; // reset timer 1 (GSCLK)
        T1TCR    = 0x01; // start timer 1 (GSCLK)
    }
    PWMIR   = 0xff;  // acknowledge interrupt
    VICVectAddr = 0x00000000;             // clear this interrupt from the VIC
}
/*
void pwm0ISR(void)
{
    if (PWMIR & PWMIR_PWMMR0_Interrupt)
    {
        FIO0PIN ^= (BLANK + XLAT); // Toggle BLANK Pin
    }
    PWMIR = 0xff; // Write back to IR to clear Interrupt Flags
    VICVectAddr = 0x0; // End of interrupt execution
}
*/
/**
 * Initialize TLC5940 driver
 **/
void TLC5940_init(void)
{
    FIO0DIR |= XLAT + BLANK + GSCLK;

    // initialize timer 1 pin MAT1.0 for GSCLK output about 7.5 MHz
    PINSEL0 |= 0x02000000; // switch pin P0.12 to Timer 1 Channel 0 match
    T1CTCR   = 0;          // set timer to timer mode (clock timer by PCLK)
    T1PR     = 1;          // set prescaler, with 60MHz PCLK this gives 15 MHz
    T1MR0    = 1;          // set compare match 1, so pin should toggle at 7,5 MHz
    T1MCR    = 2;          // set timer to reset on matching MR0
    T1EMR   |= 0x30;       // set pin mode of MAT1.0 to toggle
    T1TCR    = 0x02;       // reset timer
    T1TCR    = 0x01;       // start timer

    // initialize PWM as timer for BLANK interrupt every 4096 GSCLK pulses
    PWMPR    = 9;          // set prescaler, with 60MHz PCLK this gives 15 MHz
    PWMMR0   = 2000; //4096;       // set compare match 1, so pin should toggle at 7,5 MHz
    PWMMCR   = 3;          // set timer to reset and Interrupt on matching MR0
    PWMTCR   = 0x02;       // reset timer
    PWMTCR   = 0x01;       // start timer

    // initialize the interrupt vector
    VICIntSelect &= ~ VIC_BIT(VIC_PWM0);  // PWM0 selected as IRQ
    VICIntEnable |= VIC_BIT(VIC_PWM0);    // PWM0 interrupt enabled
    VICVectCntl5 = VIC_ENABLE | VIC_PWM0;
    VICVectAddr5 = (uint32_t)pwm0ISR;     // address of the ISR

}
